#ifndef __GLOBAL_VARIABLE_H__
#define __GLOBAL_VARIABLE_H__


#include <QString>
#include <QList>
#include <QChar>
#include <QMap>

#include <QDateTime>

/**
 * @brief 星期枚举定义，星期八是带有日期的特殊日
 * 
 */
enum class eWeekType
{
    Week_Monday = 1,    /* 星期一 */
    Week_Tuesday,       /* 星期二 */
    Week_Wednesday,     /* 星期三 */
    Week_Thursday,      /* 星期四 */
    Week_Friday,        /* 星期五 */
    Week_Saturday,      /* 星期六 */
    Week_Sunday,        /* 星期日 */
    Week_Special,       /* 特殊日期 */
};

Q_DECLARE_METATYPE(eWeekType)

const QMap<eWeekType, QString> MapWeekTypeToString = {
    {eWeekType::Week_Monday, "星期一"},
    {eWeekType::Week_Tuesday, "星期二"},
    {eWeekType::Week_Wednesday, "星期三"},
    {eWeekType::Week_Thursday, "星期四"},
    {eWeekType::Week_Friday, "星期五"},
    {eWeekType::Week_Saturday, "星期六"},
    {eWeekType::Week_Sunday, "星期日"},
    {eWeekType::Week_Special, "特殊日期"}
};


/**
 * @brief 一条计划
 * 
 */
struct OnePlan_t
{
    eWeekType weekType;     /* 星期类型 */
    QDate date;             /* 日期，格式为YYYY-MM-DD */
    QTime timeStart;        /* 时间，格式为HH:mm:ss */
    QTime timeEnd;          /* 时间，格式为HH:mm:ss */

    bool operator==(const OnePlan_t &other) const
    {
        return (weekType == other.weekType) && (date == other.date) && (timeStart == other.timeStart) && (timeEnd == other.timeEnd);
    }
    bool operator!=(const OnePlan_t &other) const
    {
        return !(*this == other);
    }
};



/* ====================================================================
 * 声卡相关信息结构体
 * ==================================================================== */

/* 单个录音通道信息，带有频道信息 */
struct OneRoadInfo_t
{
    int nRoadNum = -1;              /* 录音通道编号 */
    int nChannelID = -1;            /* 频道ID */
    QString strChannelName;         /* 频道名称 */
};


/**
 * @brief 声卡信息，这里的信息都是设备上和物理声卡相关的
 * 
 */
struct SoundCardInfo_t
{
    int nSoundCardNum = -1;             /* 声卡编号 */
    QString strSoundCardID;             /* 声卡ID，这个是声卡名称，可以用来打开 */
    QString strSoundCardName;           /* 声卡名称 */
    QString strSoundCardDriver;         /* 声卡驱动名称 */
    
    QList<OneRoadInfo_t> listRoad;      /* 录音通道列表，存储通道编号 */
};


/**
 * @brief 单个声卡通道信息，带有声卡自身的信息
 * 
 */
struct SoundCardRoadInfo_t
{
    int nSoundCardNum = -1;             /* 声卡编号 */
    QString strSoundCardID;             /* 声卡ID，这个是声卡名称，可以用来打开 */
    QString strSoundCardName;           /* 声卡名称 */
    OneRoadInfo_t roadInfo;             /* 单个通道信息 */
};

/* 注册数据类型到系统 */
Q_DECLARE_METATYPE(SoundCardRoadInfo_t)


/* ====================================================================
 * CompareItemInfo_t对比项结构体
 * ==================================================================== */

/**
 * @brief 对比项信息在表格中的显示结构体
 * 
 */
struct CompareItemTableItem_t
{
    int nSerialNum;             /* 序号，在表格中的序号 */
    int nID;                    /* 对比项ID */
    QString strName;            /* 对比项名称 */
    bool isEnable;              /* 对比项状态，是否启用 */
    int nChannelCount;          /* 通道数 */
};



/* 对比项的通道信息 */
struct CompareItemRoadInfo_t
{
    bool isEnableRecord;        /* 是否开启录音 */
    int nCompareRoadNum;        /* 对比通道编号，1是主通道，其余往后排 */
    QString strCompareRoadName; /* 通道名称 */
    SoundCardRoadInfo_t scRoadInfo; /* 声卡通道信息，包含声卡编号和通道编号 */

    bool operator==(const CompareItemRoadInfo_t &other) const;
};


/* 检测阈值，静音和过载是整数，反相是小数 */
union Threshold_t
{
    uint64_t nThreshold;  /* 静音和过载的阈值 */
    double dThreshold;    /* 反相的阈值 */
};

/**
 * @brief 静音、过载、反相检测参数
 * 
 */
struct CompareItemDetectParam_t
{
    bool isEnable = false;          /* 是否启用检测 */
    Threshold_t threshold;          /* 检测阈值 */
    int nLen = 0;                   /* 检测长度，单位秒 */
    int nSensitivity = 0;           /* 敏感度，0-100，0最不敏感，100最敏感 */

    CompareItemDetectParam_t() = default;
    CompareItemDetectParam_t(const CompareItemDetectParam_t &other);
    CompareItemDetectParam_t& operator=(const CompareItemDetectParam_t &other);
};

/**
 * @brief 对比项信息
 * 
 */
struct CompareItemInfo_t
{
    int nID = -1;                               /* 对比项ID，对比项唯一识别号，不可更改 */
    QString strName;                            /* 对比项名称 */
    bool isEnable = false;                      /* 对比项状态，是否启用 */
    QMap<int, CompareItemRoadInfo_t> mapRoad;   /* 通道信息列表，int是通道编号 */
    CompareItemDetectParam_t paramMute;         /* 静音检测参数 */
    CompareItemDetectParam_t paramOverload;     /* 过载检测参数 */
    CompareItemDetectParam_t paramPhase;        /* 反相检测参数 */

    CompareItemInfo_t() = default;
    CompareItemInfo_t(const CompareItemInfo_t &other);
    CompareItemInfo_t& operator=(const CompareItemInfo_t &other);

    /* 判断对比想基础部分是否相等，不包含通道信息 */
    bool isEqualBase(const CompareItemInfo_t &other) const;
    /* 判断对比项的通道是否相同 */
    bool isEqualRoads(const CompareItemInfo_t &other) const;
};

/* =============================================================================
 * 检测时段配置
 * ============================================================================= */

/**
 * @brief 一个对比项的检测计划
 * 
 */
struct DetectPeriodConfig_t
{
    int nID = 0;                            /* ID */
    bool isApplySlient = false;             /* 非检测计划日期是否应用静音 */
    bool isApplyOverload = false;           /* 非检测计划日期是否应用超载 */
    bool isApplyPhase = false;              /* 非检测计划日期是否应用反相 */
    bool isApplyNoise = false;              /* 非检测计划日期是否应用噪音 */
    QList<OnePlan_t> listDetect;            /* 检测计划列表 */
    QList<OnePlan_t> listNoDetect;          /* 非检测计划列表 */

    bool operator==(const DetectPeriodConfig_t &other) const;
    /* 判断基础信息是否更改 */
    bool isBaseInfoChanged(const DetectPeriodConfig_t &other) const;
    
};

/**
 * @brief 非检测计划应用清空
 * 
 */
struct NoDetectPlanApply_t
{
    int nID = 0;                            /* ID */
    bool isApplySlient = false;             /* 非检测计划日期是否应用静音 */
    bool isApplyOverload = false;           /* 非检测计划日期是否应用超载 */
    bool isApplyPhase = false;              /* 非检测计划日期是否应用反相 */
    bool isApplyNoise = false;              /* 非检测计划日期是否应用噪音 */
};




#endif // __GLOBAL_VARIABLE_H__
